package dev.fyloz.trial.colorrecipesexplorer.web.controller.files;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.RecipeService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.TouchUpKitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.servlet.ModelAndView;

import java.io.IOException;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.*;

@Controller
public class TouchUpKitController {

    private TouchUpKitService touchUpKitService;
    private RecipeService recipeService;

    @Autowired
    public TouchUpKitController(TouchUpKitService touchUpKitService, RecipeService recipeService) {
        this.touchUpKitService = touchUpKitService;
        this.recipeService = recipeService;
    }

    @GetMapping(TOUCHUP)
    public ModelAndView getPage(ModelAndView model) {
        return new ModelResponseBuilder(model)
                .withView(TOUCHUP)
                .addResponseData(ResponseDataType.RECIPE_MAP, recipeService.getRecipesByCompany())
                .build();
    }

    @PostMapping(value = TOUCHUP_PDF, produces = MediaType.APPLICATION_PDF_VALUE)
    public ResponseEntity<byte[]> getTouchUpKitPdf(@RequestBody String jobNumber) throws IOException {
        return new ResponseEntity<>(touchUpKitService.getPdfForJobNumber(jobNumber.replace("jobNumber=", "")), HttpStatus.FOUND);
    }

    @PostMapping(value = TOUCHUP_PTOUCH)
    public ModelAndView getTouchUpKitPtouch(@RequestBody String jobNumber) {
        return new ModelResponseBuilder(TOUCHUP_PTOUCH_PAGE)
                .addAttribute("jobNumber", jobNumber.replace("jobNumber=", ""))
                .build();
    }

}
