package dev.fyloz.trial.colorrecipesexplorer.web.controller.removers;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Company;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.CompanyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import java.util.Optional;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.REMOVER_COMPANY;
import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.REMOVER_COMPANY_SPECIFIC;

@Controller
public class CompanyRemoverController {

    private CompanyService companyService;

    @Autowired
    public CompanyRemoverController(CompanyService companyService) {
        this.companyService = companyService;
    }

    /**
     * Affiche la page listant toutes les bannières.
     * <p>
     * Modèle de la page:
     * - companies: Contient la liste de toutes les bannières
     *
     * @param model Le Model injecté par Thymeleaf
     * @return La page à afficher
     */
    @GetMapping(REMOVER_COMPANY)
    public ModelAndView getPage(ModelAndView model) {
        return new ModelResponseBuilder(model)
                .withView(REMOVER_COMPANY)
                .addResponseData(ResponseDataType.COMPANIES, companyService.getAll())
                .build();
    }

    /**
     * Permet à l'utilisateur de supprimer une bannière.
     * La méthode requiert l'identifiant de la bannière à supprimer dans l'URL.
     * <p>
     * La suppression échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - Il n'y a aucune bannière avec cet identifiant
     * - La bannière est associée à une ou plusieurs recettes
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - successCompanyName: Contient le nom de la bannière supprimée
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param id L'identifiant de la bannière
     * @return La page à afficher
     */

    @PostMapping(REMOVER_COMPANY_SPECIFIC)
    public ModelAndView removeCompany(@PathVariable Long id) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder("");

        Optional<Company> optionalCompany = companyService.getById(id);
        if (optionalCompany.isPresent()) {
            Company company = optionalCompany.get();

            if (companyService.deleteIfNotLinked(company))
                modelResponseBuilder.addResponseCode(ResponseCode.SUCCESS_DELETING_COMPANY, company.getName());
            else modelResponseBuilder.addResponseCode(ResponseCode.COMPANY_LINKED, company.getName());
        } else {
            modelResponseBuilder.addResponseCode(ResponseCode.COMPANY_NOT_FOUND, id);
        }

        return getPage(modelResponseBuilder.build());
    }
}
